*
* Functions to perform basic arithmetic operations on arbitrarily
* large unsigned numbers. Supports: + - * / % <<1 >>1 == != < >
*
* To work on other than 32 bits, adjust the following constants:
?LSIZE	EQU	4		4 bytes (32 bits)
$DD:Longreg	4
*
* Subtract two LONG numbers: longsub(num1, num2)
*
longsub	TSX			Address parms
	LDY	2,X		Get num2
	LDX	4,X		Get num1
?bigsub	LDAB	#?LSIZE		Get byte count
	CLC			No borrow in
?1	LDAA	,X		Get byte from num1
	SBCA	,Y		Subtract num2
	STAA	,X		Resave dest
	INX			Advance X
	INY			Advance Y
	DECB			Reduce count
	BNE	?1		Do them all
	ROLB			Get carry out
	CLRA			Zero high
	RTS
*
* Add two LONG numbers: longadd(num1, num2)
*
longadd	TSX			Address parms
	LDY	2,X		Get num2
	LDX	4,X		Get num1
?bigadd	LDAB	#?LSIZE		Get byte count
	CLC			No carry in
?2	LDAA	,X		Get byte from num1
	ADCA	,Y		Add num2
	STAA	,X		Resave dest
	INX			Advance X
	INY			Advance Y
	DECB			Reduce count
	BNE	?2		Do them all
	ROLB			Get carry out
	CLRA			Zero high
	RTS
*
* Shift LONG number right: longshr(num1)
*
longshr	TSX			Address parm
	LDX	2,X		Get num2
?bigshr	CLC			No carry in
	LDAB	#?LSIZE		Get byte count
	ABX			Adjust to end
?3	DEX			Backup X
	ROR	,X		Shift one byte
	DECB			Reduce count
	BNE	?3		Do them all
	ROLB			Get carry out
	CLRA			Zero high
	RTS
*
* Shift a LONG number left: longshl(num1)
*
longshl	TSX			Address parm
	LDX	2,X		Get num2
?bigshl	CLC			No carry in
?bigslc	LDAB	#?LSIZE		Get byte count
?4	ROL	,X		Shift one byte
	INX			Advance
	DECB			Reduce count
	BNE	?4		Do them all
	ROLB			Get carry out
	CLRA			Zero high
	RTS
*
* Divide two LONG numbers: longdiv(num1, num2)
*
longdiv	BSR	?10		Zero temp location
	LDAB	#(?LSIZE*8)+1	Get bit count
	STAB	?temp		Save for later
?5	CLC			No carry in
?6	TSX			Address parms
	LDX	4,X		Point to num1
	BSR	?bigslc		Shift it
	DEC	?temp		Reduce count
	BEQ	?14		End, exit
	LDX	#Longreg	Point to temp
	RORB			Set carry out
	BSR	?bigslc		Do shift
	TSX			Address stack
	LDY	2,X		Point to num2
	LDX	#Longreg	Point to temp
	BSR	?bigcmp		Do compare
	TSTA			Is it less
	BNE	?5		Yes, carry zero
	TSX			Address stack
	LDY	2,X		Get num2
	LDX	#Longreg	Point to temp
	BSR	?bigsub		Do subtract
	SEC			Carry in 1
	BRA	?6		And proceed
*
* Multiply two LONG numbers: longmul(num1, num2)
*
longmul	TSX			Get SP
	XGDX			in ACC D
	SUBD	#?LSIZE		Reserve space
	XGDX			Back to X
	TXS			Allocate space
	LDY	?LSIZE+2,X	Get num2 pointer
	BSR	?bigcpy		Copy the number
	BSR	?10		Zero temp location
?7	TSY			Address stack
	LDX	?LSIZE+4,Y	Point to num1
	BSR	?bigshr		Do the shift
	TSTB			Carry out?
	BEQ	?8		No, leave it
	LDX	#Longreg	Point to temp
	BSR	?bigadd		Do the addition
	TSY			Restore SP ptr
?8	LDX	?LSIZE+4,Y	Point to num1
	BSR	?bigtst		Test it
	BEQ	?9		End, exit
	TSX			Point to num3
	BSR	?bigshl		Do the shift
	TSX			Point to num3
	BSR	?bigtst		Test it
	BNE	?7		Keep going
?9	TSX			Get SP
	XGDX			into ACC D
	ADDD	#?LSIZE		Remote temporary
	XGDX			Back to ACCD
	TXS			Reset SP
	LDX	4,X		Get num1 pointer
	LDY	#Longreg	Point to temp
	BRA	?bigcpy		Copy & return
*
* Zero the temporary location
*
?10	LDX	#Longreg	Point to register
?11	LDAB	#?LSIZE		Get byte count
?12	CLR	,X		Zero one byte
	INX			Advance pointer
	DECB			Reduce count
	BNE	?12		Do them all
	RTS
*
* Test a LONG number for zero: longtst(num1)
*
longtst	TSX			Address parms
	LDX	2,X		Get num1
?bigtst	LDAA	#?LSIZE		Get byte count
?13	LDAB	,X		Get byte
	BNE	?14		Not zero - exit
	INX			Advance X
	DECA			Reduce count
	BNE	?13		Exit
?14	RTS
*
* Compare two LONG numbers: longcmp(num1, num2)
*
longcmp	TSX			Address parms
	LDY	2,X		Get num2
	LDX	4,X		Get num1
?bigcmp	LDAB	#?LSIZE		Get byte count
	ABX			Adjust num1 ptr
	ABY			Adjust num2 ptr
?15	DEX			Backup num1 ptr
	DEY			Backup num2 ptr
	LDAA	,X		Get byte from source
	SUBA	,Y		Does it match?
	BLO	?16		Its lower
	BHI	?17		Its higher
	DECB			Reduce count
	BNE	?15		Test them all
	RTS
?16	LDD	#-1		Return -1
	RTS
?17	LDD	#1		Return 1
	RTS
*
* Copy one LONG number to another: longcpy(num1, num2)
*
longcpy	TSX			Address parms
	LDY	2,X		Get num2
	LDX	4,X		Get num1
?bigcpy	LDAB	#?LSIZE		Get byte count
?18	LDAA	,Y		Get byte from num2
	STAA	,X		Copy to num1
	INX			Advance num1
	INY			Advance num2
	DECB			Reduce count
	BNE	?18		Do them all
	RTS
*
* Set a long number
*
longset	TSY			Address parms
	LDX	4,Y		Get address
	BSR	?11		Zero it
	LDX	4,Y		Get address back
	LDD	2,Y		Get value
	STAB	,X		Write LOW
	STAA	1,X		Write HIGH
	RTS
